-------------------------------------------------------------------------------
--   OCO Source Materials
--
--   Licensed Materials - Property of IBM
--
--   5724-C02
--
--   (C) Copyright IBM Corp. 2003  All Rights Reserved.
--
--   US Government Users Restricted Rights - Use, duplication or
--   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
-------------------------------------------------------------------------------
--    File Information: %W%
--    Revision: %I%
--    Date: %G% %U%
-------------------------------------------------------------------------------

--*********************************************************************************************
-----------------------------------------------------------------------------------------------
-- Program:    Clean_and_Reset_BWM_warehouse_Create_tables.sql
-- Author:     Jan Braddock
-- Run cmd:    db2 -stvf Clean_and_Reset_BWM_warehouse_Create_tables.sql
--
-- This script is the second of a two-step process to remove warehouse data for the
-- BWM wep. This script will recreate tables, indexes, sequences and triggers for the BWM 
-- wep that were removed by the first script, Clean_and_Reset_BWM_warehouse_Delete_tables.sql
--
-- This script should only be run when all the data for the BWM wep should be
-- removed from both the CDW and MART databases. The only data that will be
-- left in the CDW from this wep will be any SHARED components and their relationships.
-- For this warehouse pack the only SHARED components are the J2EE* components and
-- the IP_HOST component. Any relations for a SHARED component that had a relation
-- with BWM component will also be removed.
--
-- Limitations and concerns:
--------------------------------
-- ** This script will stop running on the first error encountered if run with:
--   db2 -stvf Clean_and_Reset_BWM_warehouse_Create_tables.sql
--
-- 1) This script should only be run when no warehouse ETL's are being to avoid
--    lock contention.
-- 2) The transactions logs must be sized large enough to handle the data deletion 
--    or the data could be rolled back after running for long periods of time.
-- 3) To improve performance after removing many rows it would be good to update
--    the database statistics.
-----------------------------------------------------------------------------------------------
--*********************************************************************************************

------------------------------------------------------------------------------------------
-- TWH_CDW
--
-- Change the TWH_CDW below to the correct CDW database name, if different.
-- *** You must change _user below to the warehouse DB2 user id for the script to run ***
------------------------------------------------------------------------------------------
CONNECT TO TWH_CDW USER _user;

COMMIT;

------------------------------------------------------------------------------------------
-- Reset extract values
------------------------------------------------------------------------------------------
UPDATE TWG.EXTRACT_CONTROL
   SET
        EXTCTL_TO_INTSEQ = -1,
        EXTCTL_FROM_INTSEQ = -1
   WHERE EXTCTL_TARGET LIKE 'BWM%';

UPDATE TWG.EXTRACT_CONTROL
   SET
        EXTCTL_FROM_DTTM = '1970-01-01-00.00.00.000000',
        EXTCTL_TO_DTTM   = '9999-01-01-00.00.00.000000'
   WHERE EXTCTL_TARGET LIKE 'BWM%';

COMMIT;

------------------------------------------------------------------------------------------
-- Clear out BWM CDW data
------------------------------------------------------------------------------------------
DELETE FROM TWG.COMPRELN
   WHERE MSRC_CORR_CD = 'BWM'; 

-- Clear out relations where a BWM component is the comp_target_id in a SHARED relation.
DELETE FROM TWG.COMPRELN
   WHERE MSRC_CORR_CD = 'SHARED' AND 
         COMP_TARGET_ID IN (SELECT COMP_ID FROM TWG.COMP WHERE MSRC_CORR_CD = 'BWM'); 

DELETE FROM TWG.COMP_EXT 
   WHERE COMP_ID IN (SELECT COMP_ID FROM TWG.COMP WHERE MSRC_CORR_CD = 'BWM');

DELETE FROM TWG.COMPATTR
   WHERE MSRC_CORR_CD = 'BWM'  AND COMP_ID NOT IN (SELECT COMP_ID  FROM TWG.COMP WHERE MSRC_CORR_CD = 'SHARED');


DELETE FROM TWG.MSMT
   WHERE MSRC_CORR_CD = 'BWM';

-- This needs cleaning up if updates have been made to the Comp table.
-- or deletes from the comp table will be constrained.
DELETE FROM TWG.COMP_CHANGE 
   WHERE COMP_ID IN (SELECT COMP_ID FROM TWG.COMP WHERE MSRC_CORR_CD = 'BWM');

-- Delete rows from the TWG.COMP in subsets to try to 
-- mitigate the impact on transaction logging.
DELETE FROM TWG.COMP 
   WHERE COMPTYP_CD = 'BWM_HOST'; 

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_TX_NODE'; 

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_TRANSACTION'; 

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_PROBE'; 

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_AGENT_GRP'; 

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_MGMT_POLICY'; 

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_MGMT_POL_GRP'; 

--------------------------------------------------
-- Recreate LONG table for COMP_ATTR_LONG
--------------------------------------------------
-- create comp attr long table and views
CREATE TABLE BWM.COMP_ATTR_LONG(
     COMPATTR_ID  INTEGER NOT NULL,
     COMP_ID      INTEGER NOT NULL,
     COMPATTR_VAL VARCHAR(4000) NOT NULL,
     AttrTyp_Cd   CHAR(17) NOT NULL
);

CREATE UNIQUE INDEX BWM.COMP_ATTR_LONG_IX
       ON BWM.COMP_ATTR_LONG (COMPATTR_ID, COMP_ID);

ALTER TABLE BWM.COMP_ATTR_LONG
       ADD CONSTRAINT ATTRL_PK
       PRIMARY KEY (COMPATTR_ID,COMP_ID);


CREATE VIEW BWM.VE_COMP_ATTR_LONG (MAX_VALUE)
     AS SELECT MAX(A.COMPATTR_ID)
     FROM TWG.CUR_COMP C,
          BWM.COMP_ATTR_LONG A
     WHERE A.COMP_ID=C.COMP_ID;


CREATE VIEW BWM.VD_COMP_ATTR_LONG(
     ORIG_COMPATTR_ID,
     ORIG_COMP_ID,
     ATTRTYP_CD,
     COMPATTR_VAL,
     ATTR_STRT_DTTM,
     ATTR_END_DTTM)
AS SELECT
     A.COMPATTR_ID,
     A.COMP_ID,
     CA.ATTRTYP_CD,
     A.COMPATTR_VAL,
     CA.COMPATTR_STRT_DTTM,
     CA.COMPATTR_END_DTTM
FROM TWG.CUR_COMP C,
     TWG.CUR_COMPATTR CA,
     BWM.COMP_ATTR_LONG A
WHERE A.COMP_ID=C.COMP_ID AND A.COMPATTR_ID=CA.COMPATTR_ID;


--------------------------------------------------
-- Recreate BWM.Exception_Log 
--------------------------------------------------
CREATE TABLE BWM.Exception_Log (
        Error_DtTm              TIMESTAMP,
        Process_Nm              VARCHAR(120),
        Step_Nm                 VARCHAR(120),
        Table_Nm                VARCHAR(120),
        Calculation             VARCHAR(300),
        Result_Value_Low        VARCHAR(300),
        Result_Value_High       VARCHAR(300),
        Actual_Value            VARCHAR(300),
        Error_Msg_Text          VARCHAR(554),
        Error_Recovery          VARCHAR(554),
        Reported_DtTm           TIMESTAMP NOT NULL
);

CREATE INDEX BWM.EXCEP_LOG_IX
       ON BWM.Exception_Log (Reported_DtTm);

------------------------------------------------------------------------------------------
-- TWH_MART
--
-- Change the TWH_MART below to the correct MART database name, if different.
--
-- *** You must change _user below to the warehouse DB2 user id for the script to run ***
------------------------------------------------------------------------------------------
CONNECT TO TWH_MART USER _user;

COMMIT;

-------------------------------------------------------
-- Prune tables
-------------------------------------------------------
CREATE TABLE BWM.PRUNE_MART_CONTROL (
    TABLE_NAME       VARCHAR(120) NOT NULL,
    PMARTC_DURATION  DECIMAL(8,0) NOT NULL
);


CREATE TABLE BWM.PRUNE_MART_LOG (
    PMART_RUN_DTTM   TIMESTAMP NOT NULL,
    TABLE_NAME       VARCHAR(30),
    PMARTC_DURATION  DECIMAL(8,0) NOT NULL,
    NUMBER_PRUNED    DECIMAL(8,0) NOT NULL
);

INSERT INTO BWM.PRUNE_MART_CONTROL VALUES ('BWM.F_TX_ND_HOUR',00000300);
INSERT INTO BWM.PRUNE_MART_CONTROL VALUES ('BWM.F_TX_ND_DAY',00000300);
INSERT INTO BWM.PRUNE_MART_CONTROL VALUES ('BWM.F_TX_ND_WEEK',00010000);
INSERT INTO BWM.PRUNE_MART_CONTROL VALUES ('BWM.F_TX_ND_MONTH',00010000);

----------------------------
--- Dimension Table:  Host
----------------------------
CREATE TABLE BWM.D_HOST(
    HOST_ID          INTEGER NOT NULL,
    HOST_NM          VARCHAR(254) NOT NULL,
    IP_ADDRESS       VARCHAR(254),
    CUST_ID          INTEGER,
    CUST_NM          VARCHAR(254),
    CENTER_NM        VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.D_HOST_IX
       ON BWM.D_HOST (HOST_ID);

ALTER TABLE BWM.D_HOST
       ADD CONSTRAINT DHOST_PK
       PRIMARY KEY (HOST_ID);

CREATE INDEX BWM.D_HOST_IX2
       ON BWM.D_HOST (HOST_NM);

--SEQUENCE
CREATE SEQUENCE BWM.HOST_ID_SEQ
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24
;

--TRIGGER
CREATE TRIGGER BWM.HOST_IL
 NO CASCADE BEFORE INSERT ON BWM.D_HOST
 REFERENCING NEW AS N FOR EACH ROW MODE DB2SQL
 SET N.HOST_ID=NEXTVAL FOR BWM.HOST_ID_SEQ
;

-- Translation table
CREATE TABLE BWM.T_HOST(
    CDW_ID           INTEGER NOT NULL,
    HOST_ID          INTEGER,
    ORIG_HOST_ID     INTEGER NOT NULL,
    HOST_NM          VARCHAR(254) NOT NULL,
    IP_ADDRESS       VARCHAR(254),
    CUST_ID          INTEGER,
    CUST_NM          VARCHAR(254),
    CENTER_NM        VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.T_HOST_IX
       ON BWM.T_HOST (ORIG_HOST_ID, CDW_ID);

ALTER TABLE BWM.T_HOST
       ADD CONSTRAINT THOST_PK
       PRIMARY KEY (ORIG_HOST_ID, CDW_ID);

-----------------------------------
--- Dimension Table:  Agent Group
-----------------------------------
CREATE TABLE BWM.D_HOST_AGENT_GRPS (
    HOST_ID          INTEGER NOT NULL,
    AGENT_GROUP      VARCHAR(250) NOT NULL
);

CREATE INDEX BWM.D_AGENT_GRPS_IX
   ON BWM.D_HOST_AGENT_GRPS (HOST_ID);

ALTER TABLE BWM.D_HOST_AGENT_GRPS
   ADD CONSTRAINT AGRP_FK FOREIGN KEY(HOST_ID) REFERENCES BWM.D_HOST(HOST_ID);

CREATE INDEX BWM.D_AGENT_GRPS_IX2
   ON BWM.D_HOST_AGENT_GRPS (AGENT_GROUP);

-- Translation table
CREATE TABLE BWM.T_HOST_AGENT_GRPS (
    CDW_ID                INTEGER NOT NULL,
    HOST_ID               INTEGER NOT NULL,
    ORIG_HOST_ID          INTEGER NOT NULL,
    ORIG_AGENT_GRP_ID     INTEGER NOT NULL,
    HOST_NAME             VARCHAR(254) NOT NULL,
    AGENT_GROUP           VARCHAR(250) NOT NULL
);

CREATE INDEX BWM.T_AGENT_GRPS_IX
   ON BWM.T_HOST_AGENT_GRPS (ORIG_AGENT_GRP_ID, CDW_ID);

-----------------------------
-- dimension table: TX
-----------------------------
CREATE TABLE BWM.D_TX(
    TX_ID            INTEGER NOT NULL,
    TX_NAME          VARCHAR(254),
    TX_DESC          VARCHAR(254),
    URL_PROTOCOL     VARCHAR(254),
    WEBSITE          VARCHAR(254),
    WEBSITE_PATH     VARCHAR(254),
    WEBSITE_QUERY    VARCHAR(254)
) 
;

CREATE UNIQUE INDEX BWM.D_TX_IX
       ON BWM.D_TX (TX_ID);

ALTER TABLE BWM.D_TX
       ADD CONSTRAINT DTX_PK
       PRIMARY KEY (TX_ID);

CREATE INDEX BWM.D_TX_IX2
       ON BWM.D_TX (TX_NAME);

--sequence
CREATE SEQUENCE BWM.TX_ID_SEQ
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--trigger
CREATE TRIGGER BWM.TX_IL
 NO CASCADE BEFORE INSERT ON BWM.D_TX
 REFERENCING NEW AS N FOR EACH ROW MODE DB2SQL
 SET N.TX_ID=NEXTVAL FOR BWM.TX_ID_SEQ
;


-- translation table
CREATE TABLE BWM.T_TX(
    CDW_ID         INTEGER NOT NULL,
    TX_ID          INTEGER NOT NULL,
    ORIG_TX_ID     INTEGER NOT NULL,
    TX_NAME        VARCHAR(254),
    TX_DESC        VARCHAR(254),
    URL_PROTOCOL   VARCHAR(254),
    WEBSITE        VARCHAR(254),
    WEBSITE_PATH   VARCHAR(254),
    WEBSITE_QUERY  VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.T_TX_IX
       ON BWM.T_TX (ORIG_TX_ID, CDW_ID);

ALTER TABLE BWM.T_TX
       ADD CONSTRAINT TTX_PK
       PRIMARY KEY (ORIG_TX_ID, CDW_ID);

------------------------
-- dimension table: APP
------------------------
CREATE TABLE BWM.D_APP(
    APP_ID           INTEGER NOT NULL,
    J2EESERVER_NAME  VARCHAR(254),
    J2EE_CELL_NAME   VARCHAR(254),
    J2EE_DOMAIN      VARCHAR(254),
    J2EE_NODE        VARCHAR(254),
    J2EE_TYPE_VER    VARCHAR(254),
    PROBE_NAME       VARCHAR(1024),
    PROBE_HOST       VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.D_APP_IX
       ON BWM.D_APP (APP_ID);

ALTER TABLE BWM.D_APP
       ADD CONSTRAINT DAPP_PK
       PRIMARY KEY (APP_ID);

--sequence
CREATE SEQUENCE BWM.APP_ID_SEQ
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--trigger
CREATE TRIGGER BWM.APP_IL
 NO CASCADE BEFORE INSERT ON BWM.D_APP
 REFERENCING NEW AS N FOR EACH ROW MODE DB2SQL
 SET N.APP_ID=NEXTVAL FOR BWM.APP_ID_SEQ
;


-- translation table
CREATE TABLE BWM.T_APP(
    CDW_ID           INTEGER NOT NULL,
    APP_ID           INTEGER,
    ORIG_APP_ID      INTEGER NOT NULL,
    J2EESERVER_NAME  VARCHAR(254),
    J2EE_CELL_NAME   VARCHAR(254),
    J2EE_DOMAIN      VARCHAR(254),
    J2EE_NODE        VARCHAR(254),
    J2EE_TYPE_VER    VARCHAR(254),
    PROBE_NAME       VARCHAR(1024),
    PROBE_HOST       VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.T_APP_IX
       ON BWM.T_APP (ORIG_APP_ID, CDW_ID);

ALTER TABLE BWM.T_APP
       ADD CONSTRAINT TAPP_PK
       PRIMARY KEY (ORIG_APP_ID, CDW_ID);

-----------------------------
-- dimension table: TX_NODE
-----------------------------
CREATE TABLE BWM.D_TX_ND(
    TX_ND_ID            INTEGER NOT NULL,
    TX_ND_NAME          VARCHAR(254),
    MANAGEMENT_POLICY   VARCHAR(254),
    INVOKING_USER       VARCHAR(254),
    CURRENT_CDW_ID      VARCHAR(254),
    ROOT_CDW_ID         VARCHAR(254),
    PARENT_CDW_ID       VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.D_TX_ND_IX
       ON BWM.D_TX_ND (TX_ND_ID);

ALTER TABLE BWM.D_TX_ND
       ADD CONSTRAINT DTXND_PK
       PRIMARY KEY (TX_ND_ID);

CREATE INDEX BWM.D_TX_ND_IX2
       ON BWM.D_TX_ND (MANAGEMENT_POLICY);

CREATE INDEX BWM.D_TX_ND_IX3
       ON BWM.D_TX_ND (INVOKING_USER);

--sequence
CREATE SEQUENCE BWM.TX_ND_ID_SEQ
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--trigger
CREATE TRIGGER BWM.TX_ND_IL
 NO CASCADE BEFORE INSERT ON BWM.D_TX_ND
 REFERENCING NEW AS N FOR EACH ROW MODE DB2SQL
 SET N.TX_ND_ID=NEXTVAL FOR BWM.TX_ND_ID_SEQ;


-- translation table
CREATE TABLE BWM.T_TX_ND(
    CDW_ID               INTEGER NOT NULL,
    TX_ND_ID             INTEGER NOT NULL,
    ORIG_TX_ND_ID        INTEGER NOT NULL,
    TX_ND_NAME           VARCHAR(254),
    MANAGEMENT_POLICY    VARCHAR(254),
    INVOKING_USER        VARCHAR(254),
    ROOT_CDW_ID          VARCHAR(254),
    PARENT_CDW_ID        VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.T_TX_ND_IX
       ON BWM.T_TX_ND (ORIG_TX_ND_ID, CDW_ID);

ALTER TABLE BWM.T_TX_ND
       ADD CONSTRAINT TTXND_PK
       PRIMARY KEY (ORIG_TX_ND_ID, CDW_ID);

------------------------------------
-- dimension table: TX_ND_POL_GRPS
------------------------------------
CREATE TABLE BWM.D_TX_ND_POL_GRPS (
    TX_ND_ID             INTEGER NOT NULL,
    POLICY_GROUP         VARCHAR(254) NOT NULL
);

CREATE INDEX BWM.D_POLICY_GRP_IX
   ON BWM.D_TX_ND_POL_GRPS (TX_ND_ID);

ALTER TABLE BWM.D_TX_ND_POL_GRPS
   ADD CONSTRAINT PGRP_FK FOREIGN KEY(TX_ND_ID) REFERENCES BWM.D_TX_ND (TX_ND_ID);

CREATE INDEX BWM.D_POLICY_GRP_IX2
   ON BWM.D_TX_ND_POL_GRPS (POLICY_GROUP);

-- translation table
CREATE TABLE BWM.T_TX_ND_POL_GRPS (
    CDW_ID                 INTEGER NOT NULL,
    TX_ND_ID               INTEGER NOT NULL,
    ORIG_TX_ND_ID          INTEGER NOT NULL,
    ORIG_POLICY_GRP_ID     INTEGER NOT NULL,
    POLICY_NAME            VARCHAR(254) NOT NULL,
    POLICY_GROUP           VARCHAR(254) NOT NULL
);

CREATE INDEX BWM.T_POLICY_GRP_IX
   ON BWM.T_TX_ND_POL_GRPS (ORIG_POLICY_GRP_ID, CDW_ID);

------------------------------------
-- dimension table: TX_NODE metric
------------------------------------
CREATE TABLE BWM.D_TX_ND_METRIC (
    METRIC_ID           INTEGER NOT NULL,
    MET_CATEGORY        VARCHAR(254),
    MET_DESC            VARCHAR(254),
    MET_NAME            VARCHAR(254),
    MET_UNITS           VARCHAR(254),
    MIN_EXISTS          CHAR(1) NOT NULL,
    MAX_EXISTS          CHAR(1) NOT NULL,
    AVG_EXISTS          CHAR(1) NOT NULL,
    TOTAL_EXISTS        CHAR(1) NOT NULL,
    MSRC_NM             VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.D_TX_ND_MET_IX
       ON BWM.D_TX_ND_METRIC (METRIC_ID);

ALTER TABLE BWM.D_TX_ND_METRIC
       ADD CONSTRAINT DTXMET_PK
       PRIMARY KEY (METRIC_ID);

--sequence
CREATE SEQUENCE BWM.TX_ND_METRIC_ID_SEQ
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--trigger
CREATE TRIGGER BWM.TX_ND_METRIC_IL
 NO CASCADE BEFORE INSERT ON BWM.D_TX_ND_METRIC
 REFERENCING NEW AS N FOR EACH ROW MODE DB2SQL
 SET N.METRIC_ID=NEXTVAL FOR BWM.TX_ND_METRIC_ID_SEQ;


-- translation table
CREATE TABLE BWM.T_TX_ND_METRIC(
    CDW_ID              INTEGER NOT NULL,
    METRIC_ID           INTEGER NOT NULL,
    ORIG_METRIC_ID      INTEGER NOT NULL,
    MET_CATEGORY        VARCHAR(254),
    MET_DESC            VARCHAR(254),
    MET_NAME            VARCHAR(254) NOT NULL,
    MET_UNITS           VARCHAR(254),
    MIN_EXISTS          CHAR(1) NOT NULL,
    MAX_EXISTS          CHAR(1) NOT NULL,
    AVG_EXISTS          CHAR(1) NOT NULL,
    TOTAL_EXISTS        CHAR(1) NOT NULL,
    MSRC_NM             VARCHAR(254)
);

CREATE UNIQUE INDEX BWM.T_TX_ND_MET_IX
       ON BWM.T_TX_ND_METRIC (ORIG_METRIC_ID, CDW_ID);

ALTER TABLE BWM.T_TX_ND_METRIC
       ADD CONSTRAINT TTXMET_PK
       PRIMARY KEY (ORIG_METRIC_ID, CDW_ID);

----------------------------
-- HOURLY fact table
----------------------------
CREATE TABLE BWM.F_TX_ND_HOUR(
    FACT_ID          INTEGER NOT NULL,
    CDW_ID           INTEGER NOT NULL,
    METRIC_ID        INTEGER NOT NULL,
    TX_ND_ID         INTEGER NOT NULL,
    TX_ID            INTEGER NOT NULL,
    APP_ID           INTEGER NOT NULL,
    HOST_ID          INTEGER NOT NULL,
    MEAS_HOUR        TIMESTAMP NOT NULL,
    MIN_VALUE        DOUBLE,
    MAX_VALUE        DOUBLE,
    AVG_VALUE        DOUBLE,
    TOTAL_VALUE      DOUBLE,
    SAMPLE_COUNT     BIGINT,
    ERROR_COUNT      BIGINT
    );

CREATE UNIQUE INDEX BWM.F_TX_ND_HR_IX
       ON BWM.F_TX_ND_HOUR (FACT_ID);

ALTER TABLE BWM.F_TX_ND_HOUR
       ADD CONSTRAINT FHOUR_PK
       PRIMARY KEY (FACT_ID);

CREATE INDEX BWM.F_TX_ND_HR_IX2
       ON BWM.F_TX_ND_HOUR (MEAS_HOUR);

CREATE INDEX BWM.F_TXND_HR1 ON BWM.F_TX_ND_HOUR(METRIC_ID);
CREATE INDEX BWM.F_TXND_HR2 ON BWM.F_TX_ND_HOUR(TX_ND_ID);
CREATE INDEX BWM.F_TXND_HR3 ON BWM.F_TX_ND_HOUR(TX_ID);
CREATE INDEX BWM.F_TXND_HR4 ON BWM.F_TX_ND_HOUR(APP_ID);
CREATE INDEX BWM.F_TXND_HR9 ON BWM.F_TX_ND_HOUR(HOST_ID);
CREATE INDEX BWM.F_TXND_HR5
    ON BWM.F_TX_ND_HOUR(METRIC_ID,TX_ND_ID,TX_ID,APP_ID,HOST_ID,MEAS_HOUR);

--SEQUENCE FOR HOURLY FACT TABLE
CREATE SEQUENCE BWM.FACT_ID_SEQ_HR
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;


--TRIGGER FOR HOURLY FACT TABLE
CREATE TRIGGER BWM.FACT_IL_HR
 NO CASCADE BEFORE INSERT ON BWM.F_TX_ND_HOUR
 REFERENCING NEW AS N FOR EACH ROW
 MODE DB2SQL
 SET N.FACT_ID=NEXTVAL FOR BWM.FACT_ID_SEQ_HR;


-- Add the required FOREIGN KEYS for the FACT table.
ALTER TABLE BWM.F_TX_ND_HOUR ADD CONSTRAINT F_HR_1
 FOREIGN KEY(METRIC_ID) REFERENCES BWM.D_TX_ND_METRIC(METRIC_ID);
ALTER TABLE BWM.F_TX_ND_HOUR ADD CONSTRAINT F_HR_2
 FOREIGN KEY(TX_ND_ID) REFERENCES BWM.D_TX_ND(TX_ND_ID);
ALTER TABLE BWM.F_TX_ND_HOUR ADD CONSTRAINT F_HR_3
 FOREIGN KEY(TX_ID) REFERENCES BWM.D_TX(TX_ID);
ALTER TABLE BWM.F_TX_ND_HOUR ADD CONSTRAINT F_HR_4
 FOREIGN KEY(APP_ID) REFERENCES BWM.D_APP(APP_ID);
ALTER TABLE BWM.F_TX_ND_HOUR ADD CONSTRAINT F_HR_9
 FOREIGN KEY(HOST_ID) REFERENCES BWM.D_HOST(HOST_ID);


-----------------------------
-- DAILY fact table
-----------------------------
CREATE TABLE BWM.F_TX_ND_DAY(
    FACT_ID       INTEGER NOT NULL,
    CDW_ID        INTEGER NOT NULL,
    METRIC_ID     INTEGER NOT NULL,
    TX_ND_ID      INTEGER NOT NULL,
    TX_ID         INTEGER NOT NULL,
    APP_ID        INTEGER NOT NULL,
    HOST_ID       INTEGER NOT NULL,
    MEAS_DATE     TIMESTAMP NOT NULL,
    MIN_VALUE     DOUBLE,
    MAX_VALUE     DOUBLE,
    AVG_VALUE     DOUBLE,
    TOTAL_VALUE   DOUBLE,
    SAMPLE_COUNT  BIGINT,
    ERROR_COUNT   BIGINT
);

CREATE UNIQUE INDEX BWM.F_TX_ND_DY_IX
       ON BWM.F_TX_ND_DAY (FACT_ID);

ALTER TABLE BWM.F_TX_ND_DAY
       ADD CONSTRAINT FDAY_PK
       PRIMARY KEY (FACT_ID);

CREATE INDEX BWM.F_TX_ND_DY_IX2
       ON BWM.F_TX_ND_DAY (MEAS_DATE);
CREATE INDEX BWM.F_TXND_DAY1 ON BWM.F_TX_ND_DAY(METRIC_ID);
CREATE INDEX BWM.F_TXND_DAY2 ON BWM.F_TX_ND_DAY(TX_ND_ID);
CREATE INDEX BWM.F_TXND_DAY3 ON BWM.F_TX_ND_DAY(TX_ID);
CREATE INDEX BWM.F_TXND_DAY4 ON BWM.F_TX_ND_DAY(APP_ID);
CREATE INDEX BWM.F_TXND_DAY9 ON BWM.F_TX_ND_DAY(HOST_ID);
CREATE INDEX BWM.F_TXND_DAY5
    ON BWM.F_TX_ND_DAY(METRIC_ID,TX_ND_ID,TX_ID,APP_ID,HOST_ID,MEAS_DATE);

--SEQUENCE FOR DAILY FACT TABLE
CREATE SEQUENCE BWM.FACT_ID_SEQ_DY
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--TRIGGER FOR DAILY FACT TABLE
CREATE TRIGGER BWM.FACT_IL_DY
 NO CASCADE BEFORE INSERT ON BWM.F_TX_ND_DAY
 REFERENCING NEW AS N FOR EACH ROW
 MODE DB2SQL
 SET N.FACT_ID=NEXTVAL FOR BWM.FACT_ID_SEQ_DY;

ALTER TABLE BWM.F_TX_ND_DAY ADD CONSTRAINT F_DAY_1
 FOREIGN KEY(METRIC_ID) REFERENCES BWM.D_TX_ND_METRIC(METRIC_ID);
ALTER TABLE BWM.F_TX_ND_DAY ADD CONSTRAINT F_DAY_2
 FOREIGN KEY(TX_ND_ID) REFERENCES BWM.D_TX_ND(TX_ND_ID);
ALTER TABLE BWM.F_TX_ND_DAY ADD CONSTRAINT F_DAY_3
 FOREIGN KEY(TX_ID) REFERENCES BWM.D_TX(TX_ID);
ALTER TABLE BWM.F_TX_ND_DAY ADD CONSTRAINT F_DAY_4
 FOREIGN KEY(APP_ID) REFERENCES BWM.D_APP(APP_ID);
ALTER TABLE BWM.F_TX_ND_DAY ADD CONSTRAINT F_DAY_9
 FOREIGN KEY(HOST_ID) REFERENCES BWM.D_HOST(HOST_ID);

-----------------------------
-- WEEKLY fact table
-----------------------------
CREATE TABLE BWM.F_TX_ND_WEEK(
    FACT_ID       INTEGER NOT NULL,
    CDW_ID        INTEGER NOT NULL,
    METRIC_ID     INTEGER NOT NULL,
    TX_ND_ID      INTEGER NOT NULL,
    TX_ID         INTEGER NOT NULL,
    APP_ID        INTEGER NOT NULL,
    HOST_ID       INTEGER NOT NULL,
    MEAS_DATE     TIMESTAMP NOT NULL,
    MIN_VALUE     DOUBLE,
    MAX_VALUE     DOUBLE,
    AVG_VALUE     DOUBLE,
    TOTAL_VALUE   DOUBLE,
    SAMPLE_COUNT  BIGINT,
    ERROR_COUNT   BIGINT
    );

CREATE UNIQUE INDEX BWM.F_TX_ND_WK_IX
       ON BWM.F_TX_ND_WEEK (FACT_ID);

ALTER TABLE BWM.F_TX_ND_WEEK
       ADD CONSTRAINT FWEEK_PK
       PRIMARY KEY (FACT_ID);

CREATE INDEX BWM.F_TX_ND_WK_IX2
       ON BWM.F_TX_ND_WEEK (MEAS_DATE);

CREATE INDEX BWM.F_TXND_WEEK1 ON BWM.F_TX_ND_WEEK(METRIC_ID);
CREATE INDEX BWM.F_TXND_WEEK2 ON BWM.F_TX_ND_WEEK(TX_ND_ID);
CREATE INDEX BWM.F_TXND_WEEK3 ON BWM.F_TX_ND_WEEK(TX_ID);
CREATE INDEX BWM.F_TXND_WEEK4 ON BWM.F_TX_ND_WEEK(APP_ID);
CREATE INDEX BWM.F_TXND_WEEK9 ON BWM.F_TX_ND_WEEK(HOST_ID);
CREATE INDEX BWM.F_TXND_WEEK5
    ON BWM.F_TX_ND_WEEK(METRIC_ID,TX_ND_ID,TX_ID,APP_ID,HOST_ID,MEAS_DATE);

--SEQUENCE FOR WEEKLY FACT TABLE
CREATE SEQUENCE BWM.FACT_ID_SEQ_WK
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--TRIGGER FOR WEEKLY FACT TABLE
CREATE TRIGGER BWM.FACT_IL_WK
 NO CASCADE BEFORE INSERT ON BWM.F_TX_ND_WEEK
 REFERENCING NEW AS N FOR EACH ROW
 MODE DB2SQL
 SET N.FACT_ID=NEXTVAL FOR BWM.FACT_ID_SEQ_WK;


ALTER TABLE BWM.F_TX_ND_WEEK ADD CONSTRAINT F_WEEK_1
 FOREIGN KEY(METRIC_ID) REFERENCES BWM.D_TX_ND_METRIC(METRIC_ID);
ALTER TABLE BWM.F_TX_ND_WEEK ADD CONSTRAINT F_WEEK_2
 FOREIGN KEY(TX_ND_ID) REFERENCES BWM.D_TX_ND(TX_ND_ID);
ALTER TABLE BWM.F_TX_ND_WEEK ADD CONSTRAINT F_WEEK_3
 FOREIGN KEY(TX_ID) REFERENCES BWM.D_TX(TX_ID);
ALTER TABLE BWM.F_TX_ND_WEEK ADD CONSTRAINT F_WEEK_4
 FOREIGN KEY(APP_ID) REFERENCES BWM.D_APP(APP_ID);
ALTER TABLE BWM.F_TX_ND_WEEK ADD CONSTRAINT F_WEEK_9
 FOREIGN KEY(HOST_ID) REFERENCES BWM.D_HOST(HOST_ID);

-----------------------------
-- MONTHLY fact table
-----------------------------
CREATE TABLE BWM.F_TX_ND_MONTH(
    FACT_ID       INTEGER NOT NULL,
    CDW_ID        INTEGER NOT NULL,
    METRIC_ID     INTEGER NOT NULL,
    TX_ND_ID      INTEGER NOT NULL,
    TX_ID         INTEGER NOT NULL,
    APP_ID        INTEGER NOT NULL,
    HOST_ID       INTEGER NOT NULL,
    MEAS_DATE     TIMESTAMP NOT NULL,
    MIN_VALUE     DOUBLE,
    MAX_VALUE     DOUBLE,
    AVG_VALUE     DOUBLE,
    TOTAL_VALUE   DOUBLE,
    SAMPLE_COUNT  BIGINT,
    ERROR_COUNT   BIGINT
    );

CREATE UNIQUE INDEX BWM.F_TX_ND_MN_IX
       ON BWM.F_TX_ND_MONTH (FACT_ID);

ALTER TABLE BWM.F_TX_ND_MONTH
       ADD CONSTRAINT FMON_PK
       PRIMARY KEY (FACT_ID);

CREATE INDEX BWM.F_TX_ND_MN_IX2
       ON BWM.F_TX_ND_MONTH (MEAS_DATE);

CREATE INDEX BWM.F_TXND_MONTH1 ON BWM.F_TX_ND_MONTH(METRIC_ID);
CREATE INDEX BWM.F_TXND_MONTH2 ON BWM.F_TX_ND_MONTH(TX_ND_ID);
CREATE INDEX BWM.F_TXND_MONTH3 ON BWM.F_TX_ND_MONTH(TX_ID);
CREATE INDEX BWM.F_TXND_MONTH4 ON BWM.F_TX_ND_MONTH(APP_ID);
CREATE INDEX BWM.F_TXND_MONTH9 ON BWM.F_TX_ND_MONTH(HOST_ID);
CREATE INDEX BWM.F_TXND_MONTH5
    ON BWM.F_TX_ND_MONTH(METRIC_ID,TX_ND_ID,TX_ID,APP_ID,HOST_ID,MEAS_DATE);

--SEQUENCE FOR MONTHLY FACT TABLE
CREATE SEQUENCE BWM.FACT_ID_SEQ_MN
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;

--TRIGGER FOR MONTHLY FACT TABLE
CREATE TRIGGER BWM.FACT_IL_MN
 NO CASCADE BEFORE INSERT ON BWM.F_TX_ND_MONTH
 REFERENCING NEW AS N FOR EACH ROW
 MODE DB2SQL
 SET N.FACT_ID=NEXTVAL FOR BWM.FACT_ID_SEQ_MN;

ALTER TABLE BWM.F_TX_ND_MONTH ADD CONSTRAINT F_MON_1
 FOREIGN KEY(METRIC_ID) REFERENCES BWM.D_TX_ND_METRIC(METRIC_ID);
ALTER TABLE BWM.F_TX_ND_MONTH ADD CONSTRAINT F_MON_2
 FOREIGN KEY(TX_ND_ID) REFERENCES BWM.D_TX_ND(TX_ND_ID);
ALTER TABLE BWM.F_TX_ND_MONTH ADD CONSTRAINT F_MON_3
 FOREIGN KEY(TX_ID) REFERENCES BWM.D_TX(TX_ID);
ALTER TABLE BWM.F_TX_ND_MONTH ADD CONSTRAINT F_MON_4
 FOREIGN KEY(APP_ID) REFERENCES BWM.D_APP(APP_ID);
ALTER TABLE BWM.F_TX_ND_MONTH ADD CONSTRAINT F_MON_9
 FOREIGN KEY(HOST_ID) REFERENCES BWM.D_HOST(HOST_ID);


-----------------------------
-- FACT DATA STAGE TABLES
-----------------------------
CREATE TABLE BWM.STG_TX_ND_HR(
    CDW_ID         INTEGER NOT NULL,
    MSMT_ID        BIGINT NOT NULL,
    ORIG_METRIC_ID INTEGER NOT NULL,
    ORIG_TX_ND_ID  INTEGER NOT NULL,
    ORIG_TX_ID     INTEGER NOT NULL,
    ORIG_APP_ID    INTEGER NOT NULL,
    ORIG_HOST_ID   INTEGER NOT NULL,
    MEAS_HOUR      TIMESTAMP NOT NULL,
    MIN_VALUE      DOUBLE,
    MAX_VALUE      DOUBLE,
    AVG_VALUE      DOUBLE,
    TOTAL_VALUE    DOUBLE,
    SAMPLE_COUNT   BIGINT,
    ERROR_COUNT    BIGINT
    );

CREATE UNIQUE INDEX BWM.STG_TXND_HR_IX
       ON BWM.STG_TX_ND_HR (CDW_ID, MSMT_ID);

ALTER TABLE BWM.STG_TX_ND_HR
       ADD CONSTRAINT S_HR_PK
       PRIMARY KEY (CDW_ID, MSMT_ID);

CREATE INDEX STG_TX_ND_HR_IX
    ON BWM.STG_TX_ND_HR(ORIG_TX_ND_ID,ORIG_TX_ID,ORIG_APP_ID,
                        ORIG_HOST_ID,ORIG_METRIC_ID,CDW_ID,MEAS_HOUR);

CREATE TABLE BWM.STG1_TX_ND_HR(
    CDW_ID           INTEGER NOT NULL,
    ORIG_METRIC_ID   INTEGER NOT NULL,
    ORIG_TX_ND_ID    INTEGER NOT NULL,
    ORIG_TX_ID       INTEGER NOT NULL,
    ORIG_APP_ID      INTEGER NOT NULL,
    ORIG_HOST_ID     INTEGER NOT NULL,
    MEAS_HOUR        TIMESTAMP NOT NULL,
    MIN_VALUE        DOUBLE,
    MAX_VALUE        DOUBLE,
    AVG_VALUE        DOUBLE,
    TOTAL_VALUE      DOUBLE,
    SAMPLE_COUNT     BIGINT,
    ERROR_COUNT      BIGINT
    );

CREATE UNIQUE INDEX BWM.STG1_TXND_HR_IX
       ON BWM.STG1_TX_ND_HR
       (ORIG_TX_ND_ID, ORIG_TX_ID, ORIG_APP_ID, ORIG_HOST_ID, ORIG_METRIC_ID, CDW_ID, MEAS_HOUR);

ALTER TABLE BWM.STG1_TX_ND_HR
       ADD CONSTRAINT S1_HR_PK
       PRIMARY KEY (ORIG_TX_ND_ID, ORIG_TX_ID, ORIG_APP_ID, ORIG_HOST_ID, ORIG_METRIC_ID, CDW_ID, MEAS_HOUR);

CREATE TABLE BWM.STG2_TX_ND_HR(
    CDW_ID           INTEGER NOT NULL,
    ORIG_METRIC_ID   INTEGER NOT NULL,
    ORIG_TX_ND_ID    INTEGER NOT NULL,
    ORIG_TX_ID       INTEGER NOT NULL,
    ORIG_APP_ID      INTEGER NOT NULL,
    ORIG_HOST_ID     INTEGER NOT NULL,
    MEAS_HOUR        TIMESTAMP NOT NULL,
    MIN_VALUE        DOUBLE,
    MAX_VALUE        DOUBLE,
    AVG_VALUE        DOUBLE,
    TOTAL_VALUE      DOUBLE,
    SAMPLE_COUNT     BIGINT,
    ERROR_COUNT      BIGINT
    );

CREATE UNIQUE INDEX BWM.STG2_TXND_HR_IX
       ON BWM.STG2_TX_ND_HR
       (ORIG_TX_ND_ID, ORIG_TX_ID, ORIG_APP_ID, ORIG_HOST_ID, ORIG_METRIC_ID, CDW_ID, MEAS_HOUR);

ALTER TABLE BWM.STG2_TX_ND_HR
       ADD CONSTRAINT S2_HR_PK
       PRIMARY KEY (ORIG_TX_ND_ID, ORIG_TX_ID, ORIG_APP_ID, ORIG_HOST_ID, ORIG_METRIC_ID, CDW_ID, MEAS_HOUR);

--SEQUENCE
CREATE SEQUENCE BWM.STG_MSMT_ID_SEQ
 START WITH 0 INCREMENT BY 1 NO MAXVALUE NO CYCLE CACHE 24;


-----------------------------------------------------------------
-- Create LONG tables for comp and compattr in the MART database
-----------------------------------------------------------------
-- create comp name long table
CREATE TABLE BWM.COMP_NAME_LONG (
     COMP_ID         INTEGER NOT NULL,
     ORIG_COMP_ID    INTEGER NOT NULL,
     COMP_NM         VARCHAR(4000),
     COMP_DS         VARCHAR(254),
     COMP_SHORT_NM   VARCHAR(254),
     COMPTYP_CD      VARCHAR(17),
     COMP_STRT_DTTM  TIMESTAMP NOT NULL,
     COMP_END_DTTM   TIMESTAMP NOT NULL
);

CREATE UNIQUE INDEX BWM.COMP_NM_LONG_IX
       ON BWM.COMP_NAME_LONG (ORIG_COMP_ID);

ALTER TABLE BWM.COMP_NAME_LONG
       ADD CONSTRAINT COMPL_PK
       PRIMARY KEY (ORIG_COMP_ID);

-- Recreate comp attr long table
CREATE TABLE BWM.COMP_ATTR_LONG (
     COMP_ID           INTEGER NOT NULL,
     ORIG_COMPATTR_ID  INTEGER NOT NULL,
     ORIG_COMP_ID      INTEGER NOT NULL,
     ATTRTYP_CD        VARCHAR(17) NOT NULL,
     COMPATTR_VAL      VARCHAR(4000),
     ATTR_STRT_DTTM    TIMESTAMP NOT NULL,
     ATTR_END_DTTM     TIMESTAMP NOT NULL
);

CREATE UNIQUE INDEX BWM.COMP_ATR_LONG_IX
       ON BWM.COMP_ATTR_LONG (ORIG_COMPATTR_ID);

ALTER TABLE BWM.COMP_ATTR_LONG
       ADD CONSTRAINT ATTRLL_PK
       PRIMARY KEY (ORIG_COMPATTR_ID);

--------------------------------------------------------------------------------
-- This table stores any error exceptions generated during extract processing
--------------------------------------------------------------------------------
CREATE TABLE BWM.Exception_Log (
        Error_DtTm              TIMESTAMP,
        Process_Nm              VARCHAR(120),
        Step_Nm                 VARCHAR(120),
        Table_Nm                VARCHAR(120),
        Calculation             VARCHAR(300),
        Result_Value_Low        VARCHAR(300),
        Result_Value_High       VARCHAR(300),
        Actual_Value            VARCHAR(300),
        Error_Msg_Text          VARCHAR(554),
        Error_Recovery          VARCHAR(554),
        Reported_DtTm           TIMESTAMP NOT NULL
);


CREATE INDEX BWM.EXCEP_LOG_IX
       ON BWM.Exception_Log (Reported_DtTm);

-- PlaceHolder Cust views for hourly fact table and host dimension table
-- Use these view to do SQL queries based on the cust_id. See the
-- Enablement Guide for more information about setting this and additional
-- views for other cust_id's to do queries from the data mart based on cust_id

CREATE VIEW BWM.F_V_TX_ND_999_HOUR as
  (SELECT * FROM BWM.F_TX_ND_HOUR
     WHERE host_id IN
        (SELECT host_id FROM BWM.D_HOST WHERE cust_id = 999));

CREATE VIEW BWM.D_V_HOST_999 as
  (SELECT * FROM BWM.D_HOST WHERE cust_id = 999);

-------------------------------------------------------------
-- Create the BWM hourly view for the Crystal reports
-------------------------------------------------------------

-- Create the view of the BWM hourly fact table to mask the
-- column name difference of meas_hour where the other fact
-- tables have meas_date
CREATE VIEW BWM.F_TX_ND_HOUR_VIEW (
    FACT_ID,
    CDW_ID,
    METRIC_ID,
    TX_ND_ID,
    TX_ID,
    APP_ID,
    HOST_ID,
    MEAS_DATE,
    MIN_VALUE,
    MAX_VALUE,
    AVG_VALUE,
    TOTAL_VALUE,
    SAMPLE_COUNT,
    ERROR_COUNT ) AS
SELECT
    FACT_ID,
    CDW_ID,
    METRIC_ID,
    TX_ND_ID,
    TX_ID,
    APP_ID,
    HOST_ID,
    MEAS_HOUR,
    MIN_VALUE,
    MAX_VALUE,
    AVG_VALUE,
    TOTAL_VALUE,
    SAMPLE_COUNT,
    ERROR_COUNT
FROM BWM.F_TX_ND_HOUR;

